"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Mixpanel = require("mixpanel");
const fs = require("fs-extra");
const path = require("path");
const promptly = require("promptly");
const mixpanel = Mixpanel.init('953cbc7bc5e8daa02b91425ac92d36dd');
const DEFAULT_PATH = path.join(__dirname, '../realm-object-server-agreement.json');
function writeAgreementJsonToProcessWorkingDirectory(email, agreeToLearnAboutUpdatesAndFixes) {
    return __awaiter(this, void 0, void 0, function* () {
        const json = {
            email: email,
            agreeToLearnAboutUpdatesAndFixes
        };
        yield fs.writeFile(DEFAULT_PATH, JSON.stringify(json, null, 4), { encoding: 'utf8' });
    });
}
function attemptToSubmitToMixpanel(email, agreeToLearnAboutUpdatesAndFixes) {
    return __awaiter(this, void 0, void 0, function* () {
        mixpanel.track('ros_agreed_to_terms', {
            version: (yield fs.readJson(path.join(__dirname, '../package.json'))).version,
            email: email,
            agreeToLearnAboutUpdatesAndFixes: agreeToLearnAboutUpdatesAndFixes
        });
    });
}
function validateEmail(email) {
    const re = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
    const passesTest = re.test(email);
    if (!passesTest) {
        throw new Error('Please enter a valid email');
    }
    return email;
}
function validate(email) {
    const re = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
    const passesTest = re.test(email);
    if (!passesTest) {
        throw new Error('Please enter a valid email');
    }
    return email;
}
function agreeToTerms() {
    return __awaiter(this, void 0, void 0, function* () {
        if (process.env.JENKINS || process.env.ROS_SKIP_PROMTS) {
            console.log(`SKIPPING AGREEMENT TO TERMS DUE TO ENVIRONMENT VARIABLE.`);
            return;
        }
        const jsonExists = yield fs.pathExists(DEFAULT_PATH);
        if (jsonExists) {
            return;
        }
        console.log('Please agree to our terms of service by siging with your email. You can read the full terms at: https://realm.io/legal/developer-license-terms/');
        const email = yield promptly.prompt(`Agree with your email: `, { validator: validateEmail });
        const agreeToLearnAboutUpdatesAndFixes = yield promptly.confirm('Would you like to receive product updates and updates about critical bug fixes? (Y/N)');
        try {
            yield attemptToSubmitToMixpanel(email, agreeToLearnAboutUpdatesAndFixes);
        }
        catch (err) {
        }
        yield writeAgreementJsonToProcessWorkingDirectory(email, agreeToLearnAboutUpdatesAndFixes);
    });
}
exports.agreeToTerms = agreeToTerms;
//# sourceMappingURL=termsofservice.js.map